Class: IO
- Inherits:
-
Object
- Object
- IO
- Defined in:
- sig/shims/env.rbs
Class Method Summary collapse
-
.popen ⇒ Object
IO.popen([env_hash, cmd, *args])— the env-hash-inside-the-array form the boot probe uses; rbs core only types env as a separate leading argument.
Class Method Details
.popen ⇒ void .popen ⇒ void
IO.popen([env_hash, cmd, *args]) — the env-hash-inside-the-array form
the boot probe uses; rbs core only types env as a separate leading
argument.
17 18 |
# File 'sig/shims/env.rbs', line 17
def self.popen: [X] (Array[Hash[String, String?] | String] env_and_cmd, ?String mode) { (instance) -> X } -> X
| ...
|