Class: Ryac::LocalScopes::Scope
- Inherits:
-
Struct
- Object
- Struct
- Ryac::LocalScopes::Scope
- Defined in:
- sig/ryac/analysis/local_scopes.rbs,
lib/ryac/analysis/local_scopes.rb
Overview
Runtime class is built via Struct.new(..., keyword_init: true).
Instance Attribute Summary collapse
-
#id ⇒ Object
Returns the value of attribute id.
-
#kind ⇒ Object
Returns the value of attribute kind.
-
#mapping ⇒ Object
Returns the value of attribute mapping.
-
#node ⇒ Object
Returns the value of attribute node.
-
#owner_call ⇒ Object
Returns the value of attribute owner_call.
-
#parent ⇒ Object
Returns the value of attribute parent.
Class Method Summary collapse
Instance Method Summary collapse
-
#allocated? ⇒ Boolean
declared on the Scope class itself in the RBS.
Instance Attribute Details
#id ⇒ Object
Returns the value of attribute id
34 35 36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 34 def id @id end |
#kind ⇒ Object
Returns the value of attribute kind
34 35 36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 34 def kind @kind end |
#mapping ⇒ Object
Returns the value of attribute mapping
34 35 36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 34 def mapping @mapping end |
#node ⇒ Object
Returns the value of attribute node
34 35 36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 34 def node @node end |
#owner_call ⇒ Object
Returns the value of attribute owner_call
34 35 36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 34 def owner_call @owner_call end |
#parent ⇒ Object
Returns the value of attribute parent
34 35 36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 34 def parent @parent end |
Class Method Details
.new ⇒ Scope
18 |
# File 'sig/ryac/analysis/local_scopes.rbs', line 18
def self.new: (?id: scope_id, ?kind: Symbol, ?node: prism_scope_node, ?parent: Scope?, ?owner_call: Prism::CallNode?, ?mapping: Hash[Symbol, String]?) -> Scope
|
Instance Method Details
#allocated? ⇒ Boolean
declared on the Scope class itself in the RBS
36 |
# File 'lib/ryac/analysis/local_scopes.rb', line 36 def allocated? = !mapping.nil? # steep:ignore UndeclaredMethodDefinition, NoMethod |