Class: Ryac::Pipeline::FileEntry

Inherits:
Data
  • Object
show all
Defined in:
lib/ryac/pipeline/data_types.rb,
sig/ryac/pipeline/data_types.rbs

Overview

Represents a single source file discovered during collection Immutable after creation

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(path:, content:, dependencies:, require_nodes:, in_class_dependencies: [], lazy: false) ⇒ FileEntry

declared on the Data class itself in the RBS

Parameters:

  • path: (String)
  • content: (String)
  • dependencies: (Array[String])
  • require_nodes: (Array[require_node_info])
  • in_class_dependencies: (Array[String]) (defaults to: [])
  • lazy: (Boolean) (defaults to: false)


16
17
18
# File 'lib/ryac/pipeline/data_types.rb', line 16

def initialize(path:, content:, dependencies:, require_nodes:, in_class_dependencies: [], lazy: false) # steep:ignore UndeclaredMethodDefinition
  super
end

Instance Attribute Details

#content ⇒ Object (readonly)

Returns the value of attribute content

Returns:

  • (Object) —

    the current value of content



7
8
9
# File 'lib/ryac/pipeline/data_types.rb', line 7

def content
  @content
end

#dependencies ⇒ Object (readonly)

Returns the value of attribute dependencies

Returns:

  • (Object) —

    the current value of dependencies



7
8
9
# File 'lib/ryac/pipeline/data_types.rb', line 7

def dependencies
  @dependencies
end

#in_class_dependencies ⇒ Object (readonly)

Returns the value of attribute in_class_dependencies

Returns:

  • (Object) —

    the current value of in_class_dependencies



7
8
9
# File 'lib/ryac/pipeline/data_types.rb', line 7

def in_class_dependencies
  @in_class_dependencies
end

#lazy ⇒ Object (readonly)

Returns the value of attribute lazy

Returns:

  • (Object) —

    the current value of lazy



7
8
9
# File 'lib/ryac/pipeline/data_types.rb', line 7

def lazy
  @lazy
end

#path ⇒ Object (readonly)

Returns the value of attribute path

Returns:

  • (Object) —

    the current value of path



7
8
9
# File 'lib/ryac/pipeline/data_types.rb', line 7

def path
  @path
end

#require_nodes ⇒ Object (readonly)

Returns the value of attribute require_nodes

Returns:

  • (Object) —

    the current value of require_nodes



7
8
9
# File 'lib/ryac/pipeline/data_types.rb', line 7

def require_nodes
  @require_nodes
end

Class Method Details

.new ⇒ FileEntry

Parameters:

  • path: (String)
  • content: (String)
  • dependencies: (Array[String])
  • require_nodes: (Array[require_node_info])
  • in_class_dependencies: (Array[String])
  • lazy: (Boolean)

Returns:



11
# File 'sig/ryac/pipeline/data_types.rbs', line 11

def self.new: (path: String, content: String, dependencies: Array[String], require_nodes: Array[require_node_info], ?in_class_dependencies: Array[String], ?lazy: bool) -> FileEntry