Class: Ryac::Pipeline::FileEntry
- Inherits:
-
Data
- Object
- Data
- Ryac::Pipeline::FileEntry
- Defined in:
- lib/ryac/pipeline/data_types.rb,
sig/ryac/pipeline/data_types.rbs
Overview
Represents a single source file discovered during collection Immutable after creation
Instance Attribute Summary collapse
-
#content ⇒ Object
readonly
Returns the value of attribute content.
-
#dependencies ⇒ Object
readonly
Returns the value of attribute dependencies.
-
#in_class_dependencies ⇒ Object
readonly
Returns the value of attribute in_class_dependencies.
-
#lazy ⇒ Object
readonly
Returns the value of attribute lazy.
-
#path ⇒ Object
readonly
Returns the value of attribute path.
-
#require_nodes ⇒ Object
readonly
Returns the value of attribute require_nodes.
Class Method Summary collapse
Instance Method Summary collapse
-
#initialize(path:, content:, dependencies:, require_nodes:, in_class_dependencies: [], lazy: false) ⇒ FileEntry
constructor
declared on the Data class itself in the RBS.
Constructor Details
#initialize(path:, content:, dependencies:, require_nodes:, in_class_dependencies: [], lazy: false) ⇒ FileEntry
declared on the Data class itself in the RBS
16 17 18 |
# File 'lib/ryac/pipeline/data_types.rb', line 16 def initialize(path:, content:, dependencies:, require_nodes:, in_class_dependencies: [], lazy: false) # steep:ignore UndeclaredMethodDefinition super end |
Instance Attribute Details
#content ⇒ Object (readonly)
Returns the value of attribute content
7 8 9 |
# File 'lib/ryac/pipeline/data_types.rb', line 7 def content @content end |
#dependencies ⇒ Object (readonly)
Returns the value of attribute dependencies
7 8 9 |
# File 'lib/ryac/pipeline/data_types.rb', line 7 def dependencies @dependencies end |
#in_class_dependencies ⇒ Object (readonly)
Returns the value of attribute in_class_dependencies
7 8 9 |
# File 'lib/ryac/pipeline/data_types.rb', line 7 def in_class_dependencies @in_class_dependencies end |
#lazy ⇒ Object (readonly)
Returns the value of attribute lazy
7 8 9 |
# File 'lib/ryac/pipeline/data_types.rb', line 7 def lazy @lazy end |
#path ⇒ Object (readonly)
Returns the value of attribute path
7 8 9 |
# File 'lib/ryac/pipeline/data_types.rb', line 7 def path @path end |
#require_nodes ⇒ Object (readonly)
Returns the value of attribute require_nodes
7 8 9 |
# File 'lib/ryac/pipeline/data_types.rb', line 7 def require_nodes @require_nodes end |
Class Method Details
.new ⇒ FileEntry
11 |
# File 'sig/ryac/pipeline/data_types.rbs', line 11
def self.new: (path: String, content: String, dependencies: Array[String], require_nodes: Array[require_node_info], ?in_class_dependencies: Array[String], ?lazy: bool) -> FileEntry
|